/**
 * Extend the basic ActorSheet with some very simple modifications
 * @extends {ActorSheet}
 */

import { RdDUtility } from "./rdd-utility.js";
import { RdDEncaisser } from "./rdd-roll-encaisser.js";
 
/* -------------------------------------------- */
export class RdDActorSheet extends ActorSheet {

  /** @override */
	static get defaultOptions() {
	  return mergeObject(super.defaultOptions, {
  	  classes: ["rdd", "sheet", "actor"],
  	  template: "systems/foundryvtt-reve-de-dragon/templates/actor-sheet.html",
      width: 640,
      height: 720,
      tabs: [{navSelector: ".sheet-tabs", contentSelector: ".sheet-body", initial: "carac"}],
      dragDrop: [{dragSelector: ".item-list .item", dropSelector: null}]
    });
  }

  /* -------------------------------------------- */
  _checkNull(items) {
    if (items && items.length) {
      return items;
    }
    return [];
  }

  /* -------------------------------------------- */
  getData() {
    let data = super.getData();
    
    data.itemsByType = {};
    for (const item of data.items) {
      let list = data.itemsByType[item.type];
      if (!list) {
        list = [];
        data.itemsByType[item.type] = list;
      }
      list.push(item);
    }
    // Competence per category
    data.competenceByCategory = {};
    let competenceXPTotal = 0;
    if (data.itemsByType.competence) { 
      for (const item of data.itemsByType.competence) {
        //console.log("Push...", item, item.data.categorie);
        let list = data.competenceByCategory[item.data.categorie];
        if (!list) {
          list = [];
          data.competenceByCategory[item.data.categorie] = list;
        }
        if (!RdDUtility.isTronc( item.name ) ) // Ignorer les compétences 'troncs' à ce stade
          competenceXPTotal += RdDUtility.computeCompetenceXPCost(item)
        list.push(item);
      }
    }
    competenceXPTotal += RdDUtility.computeCompetenceTroncXP(data.itemsByType.competence);
    data.data.competenceXPTotal = competenceXPTotal;
    //console.log("XP competence : ", competenceXPTotal);

    // Compute current carac sum
    let sum = 0;
    Object.values(data.data.carac).forEach(carac => { if (!carac.derivee) { sum += parseInt(carac.value) } } );
    data.data.caracSum = sum;

    // Force empty arme, at least for Esquive
    if (data.itemsByType.arme == undefined ) data.itemsByType.arme = [];
    for (const arme of data.itemsByType.arme) {
      arme.data.niveau = 0; // Per default, TODO to be fixed
      for ( const melee of data.competenceByCategory.melee ) {
        if (melee.name == arme.data.competence ) 
          arme.data.niveau = melee.data.niveau
      }
      for ( const tir of data.competenceByCategory.tir ) {
        if (tir.name == arme.data.competence ) 
          arme.data.niveau = tir.data.niveau
      }
      for ( const lancer of data.competenceByCategory.lancer ) {
        if (lancer.name == arme.data.competence ) 
          arme.data.niveau = lancer.data.niveau
      }
    }
    
    // To avoid armour and so on...
    data.data.combat = duplicate( this._checkNull(data.itemsByType['arme']));

    if (data.competenceByCategory && data.competenceByCategory.melee) { 
      //Specific case for Esquive and Corps à Corps
      for ( const melee of data.competenceByCategory.melee ) {
        if (melee.name == "Esquive")
          data.data.combat.push( { name: "Esquive", data: { niveau: melee.data.niveau, description: "", force: 6, competence: "Esquive", dommages: 0} } );
        if (melee.name == "Corps à corps")
          data.data.combat.push( { name: "Corps à corps", data: { niveau: melee.data.niveau, description: "", force: 6, competence: "Corps à corps", dommages: data.data.attributs.plusdom.value } } );
      }
    }
    
    data.data.carac.taille.isTaille = true; // To avoid button link;
    data.data.nbLegeres   = this.actor.GetNumberBlessures(data.data.blessures.legeres.liste );
    data.data.nbGraves    = this.actor.GetNumberBlessures(data.data.blessures.graves.liste );
    data.data.nbCritiques = this.actor.GetNumberBlessures(data.data.blessures.critiques.liste );
    
    // Mise à jour de l'encombrement total
    this.actor.computeEncombrementTotal();

    // Common data
    data.data.competenceByCategory = data.competenceByCategory;
    data.data.encombrementTotal = this.actor.encombrementTotal;
    data.data.isGM = game.user.isGM;
    data.ajustementsConditions = CONFIG.RDD.ajustementsConditions;
    data.difficultesLibres = CONFIG.RDD.difficultesLibres;

    // low is normal, this the base used to compute the grid.
    data.data.fatigueHTML = "<table class='table-fatigue'>" + RdDUtility.makeHTMLfatigueMatrix( data.data.sante.fatigue.value,  data.data.sante.endurance.max ).html() + "</table>";
    data.data.materiel    = this._checkNull(data.itemsByType['objet']);
    data.data.conteneurs  = this._checkNull(data.itemsByType['conteneur']);
    data.data.armes       = this._checkNull(data.itemsByType['arme']);
    data.data.armures     = this._checkNull(data.itemsByType['armure']);
    data.data.livres      = this._checkNull(data.itemsByType['livre']);
    data.data.potions     = this._checkNull(data.itemsByType['potions']);
    data.data.ingredients = this._checkNull(data.itemsByType['ingredient']);
    data.data.munitions   = this._checkNull(data.itemsByType['munition']);
    data.data.herbes      = this._checkNull(data.itemsByType['herbe']);
    data.data.sorts       = this._checkNull(data.itemsByType['sort']);
    data.data.queues      = this._checkNull(data.itemsByType['queue']);
    data.data.souffles    = this._checkNull(data.itemsByType['souffle']);
    data.data.ombres      = this._checkNull(data.itemsByType['ombre']);
    data.data.tetes       = this._checkNull(data.itemsByType['tete']);
    data.data.objets      = data.data.conteneurs.concat(data.data.materiel).concat(data.data.armes).concat(data.data.armures).concat(data.data.munitions).concat(data.data.livres).concat(data.data.potions).concat(data.data.herbes).concat(data.data.ingredients);
    
    this.objetVersConteneur = {}; // Table de hash locale pour recupération rapide du conteneur parent (si existant)    
    // Attribution des objets aux conteneurs
    for (let conteneur of data.data.conteneurs) {
      conteneur.subItems = [];
      conteneur.encTotal = 0;
      if (conteneur.data.contenu) {
        for (let id of conteneur.data.contenu) {
          let objet = data.data.objets.find( objet => (id == objet._id) );
          if (objet) {
            objet.estContenu = true; // Permet de filtrer ce qui est porté dans le template
            this.objetVersConteneur[id] = conteneur._id;
            conteneur.encTotal += objet.data.encombrement;
            conteneur.subItems.push( objet );
          }
        }
      }
    }
    // Construit la liste des conteneurs de niveau 1 (c'est à dire non contenu eux-même dans un conteneur)
    let newConteneurs = data.data.conteneurs.filter(function(conteneur, index, arr) { return !conteneur.estContenu } );
    data.data.conteneurs = newConteneurs;

    return data;
  }
  
  /* -------------------------------------------- */
  async displayDialogEncaisser( ) {
    let data = { ajustementsConditions: CONFIG.RDD.ajustementsConditions /* techniquement on peut encaisser à +20 */ };
    let html = await renderTemplate('systems/foundryvtt-reve-de-dragon/templates/dialog-roll-encaisser.html', data );
    new RdDEncaisser(html, this.actor ).render(true);
  }

  /* -------------------------------------------- */
  async _onDrop(event) {
    let dragData = JSON.parse(event.dataTransfer.getData("text/plain"));
    let dropID = $(event.target).parents(".item").attr("data-item-id"); // Only relevant if container drop
    if ( dropID ) { // Dropped over an item !!!
      let objetId = dragData.id || dragData.data._id;
      if ( this.objetVersConteneur[objetId] != dropID ) {
        if ( this.actor.testConteneurCapacite(objetId, dropID) ) { 
          this.actor.enleverDeConteneur(objetId, this.objetVersConteneur[objetId]);
          this.actor.ajouterAConteneur(objetId, dropID);
        } else {
          ui.notifications.info("Capacité d'encombrement insuffisante dans le conteneur !");
        }
      }
    }
    console.log("DATA !!!!", dragData, dropID );
    super._onDrop(event)
    this.actor.computeEncombrementTotal();
  }

  /* -------------------------------------------- */
  async confirmerSuppression(li) {
    let itemId = li.data("item-id");
    let d = new Dialog({
      title: "Confirmer la suppression",
      content: "<p>Etes vous certain de vouloir supprimer cet objet ?</p>",
      buttons: {
        delete: {
          icon: '<i class="fas fa-check"></i>',
          label: "Supprimer",
          callback: () => { 
            this.actor.deleteOwnedItem(  itemId );
            li.slideUp(200, () => this.render(false));    
          }
        },
        cancel: {
          icon: '<i class="fas fa-times"></i>',
          label: "Annuler"
        }
      },
      default: "cancel",
    });
    d.render(true);
  }

  /* -------------------------------------------- */
  /** @override */
	activateListeners(html) {
    super.activateListeners(html);
    
    // Everything below here is only needed if the sheet is editable
    if (!this.options.editable) return;

    // Update Inventory Item
    html.find('.item-edit').click(ev => {
      const li = $(ev.currentTarget).parents(".item");
      const item = this.actor.getOwnedItem(li.data("item-id"));
      item.sheet.render(true);
    });
    
    // Delete Inventory Item
    html.find('.item-delete').click(ev => {
      const li = $(ev.currentTarget).parents(".item");
      this.confirmerSuppression( li );
    });

    // Update Inventory Item
    html.find('.encaisser-direct').click(ev => {
      this.displayDialogEncaisser()
    });

    // Blessure control
    html.find('.blessure-control').click(ev => {
      const li   = $(ev.currentTarget).parents(".item");
      let btype  = li.data("blessure-type");
      let index  = li.data('blessure-index');
      let active = $(ev.currentTarget).data('blessure-active');
      //console.log(btype, index, active);
      this.actor.manageBlessureFromSheet(btype, index, active).then( this.render(true) );
    });

    // Blessure data
    html.find('.blessures-soins').change(ev => {
      const li   = $(ev.currentTarget).parents(".item");
      let btype  = li.data('blessure-type');
      let index  = li.data('blessure-index');
      let psoins    = li.find('input[name=premiers_soins]').val();
      let pcomplets = li.find('input[name=soins_complets]').val();
      let jours     = li.find('input[name=jours]').val();
      let loc       = li.find('input[name=localisation]').val();
      //console.log(btype, index, psoins, pcomplets, jours, loc);
      this.actor.setDataBlessureFromSheet(btype, index, psoins, pcomplets, jours, loc).then( this.render(true) );
    });

    // Equip Inventory Item
    html.find('.item-equip').click(ev => {
      const li = $(ev.currentTarget).parents(".item");
      this.actor.equiperObjet(li.data("item-id"));
      this.render(true);
    });

    // Roll Carac
    html.find('.carac-label a').click((event) => {
      let caracName = event.currentTarget.attributes.name.value;
      this.actor.rollCarac( caracName.toLowerCase() );
    });

    // Roll Skill
    html.find('.competence-label a').click((event) => {
      let compName = event.currentTarget.text;
      this.actor.rollCompetence( compName);
    });
    // Points de reve actuel
    html.find('.ptreve-actuel a').click((event) => {
      this.actor.rollCarac( 'reveActuel' );
    });
    
    // Roll Weapon1
    html.find('.arme-label a').click((event) => {
      let armeName = event.currentTarget.text;
      this.actor.rollArme( armeName);
    });
    // Display TMR, normal
    html.find('.visu-tmr a').click((event) => {
      this.actor.displayTMR( "visu");
    });

    // Display TMR, normal
    html.find('.monte-tmr a').click((event) => {
      this.actor.displayTMR( "normal" );
    });
    
    // Display TMR, fast 
    html.find('.monte-tmr-rapide a').click((event) => {
      this.actor.displayTMR( "rapide" );
    });

    // Display info about queue
    html.find('.queuesouffle-label a').click((event) => {
      let myID = event.currentTarget.attributes['data-item-id'].value;
      const item = this.actor.getOwnedItem(myID);
      item.sheet.render(true);
    });
    // Display info about queue
    html.find('.sort-label a').click((event) => {
      let myID = event.currentTarget.attributes['data-id'].value;
      const item = this.actor.getOwnedItem(myID);
      item.sheet.render(true);
    });

    // On carac change
    html.find('.carac-value').change((event) => {
      let caracName = event.currentTarget.name.replace(".value", "").replace("data.carac.", "");
      //console.log("Value changed :", event, caracName);
      this.actor.updateCarac( caracName, parseInt(event.target.value) );
    } );
    
    // On competence change
    html.find('.competence-value').change((event) => {
      let compName = event.currentTarget.attributes.compname.value;
      //console.log("Competence changed :", compName);
      this.actor.updateCompetence( compName, parseInt(event.target.value) );
    } );    
    
    // On pts de reve change
    html.find('.pointsreve-value').change((event) => {
      let reveValue = event.currentTarget.value;
      let reve = duplicate( this.actor.data.data.reve.reve );
      reve.value = reveValue;
      this.actor.update( { "data.reve.reve": reve } );
    } );

    // On competence change
    html.find('.competence-xp').change((event) => {
      let compName = event.currentTarget.attributes.compname.value;
      this.actor.updateCompetenceXP( compName, parseInt(event.target.value) );
    } );

    // On stress change
    html.find('.compteur-edit').change((event) => {
      let fieldName = event.currentTarget.attributes.name.value;
      this.actor.updateCompteurValue( fieldName, parseInt(event.target.value) );
    } );
    
    html.find('#stress-test').click((event) => {
      this.actor.stressTest();
      this.render(true);
    });
    
    html.find('#vie-plus').click((event) => {
      this.actor.santeIncDec("vie", 1);
      this.render(true);
    });
    html.find('#vie-moins').click((event) => {
      this.actor.santeIncDec("vie", -1);
      this.render(true);
    });
    html.find('#endurance-plus').click((event) => {
      this.actor.santeIncDec("endurance", 1);
      this.render(true);
    });
    html.find('#endurance-moins').click((event) => {
      this.actor.santeIncDec("endurance", -1);
      this.render(true);
    });
    html.find('#fatigue-plus').click((event) => {
      this.actor.santeIncDec("fatigue", 1);
      this.render(true);
    });
    html.find('#fatigue-moins').click((event) => {
      this.actor.santeIncDec("fatigue", -1);
      this.render(true);
    });
  }
  

  /* -------------------------------------------- */

  /** @override */
  setPosition(options={}) {
    const position = super.setPosition(options);
    const sheetBody = this.element.find(".sheet-body");
    const bodyHeight = position.height - 192;
    sheetBody.css("height", bodyHeight);
    return position;
  }


  /* -------------------------------------------- */

  /** @override */
  _updateObject(event, formData) {
    // Update the Actor
    return this.object.update(formData);
  }
}
